%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief: Calculates the Backward-Gaussian Elimination
% This method can be used for Matrix Inversion,or direct solving of equations
% Basically creates a result in the form of:
% D 0 0 0 0
% 0 D 0 0 0
% 0 0 D 0 0
% 0 0 0 D 0
% 0 0 0 0 D
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Y ] = GaussElimBack( X )

%%%%%%%%%%%%%%%%%%%%%%%
% Grab Size of Matrix %
[Ny,~] = size(X);
Y = X;
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Backward Gaussian Substituation %
for Stage = 2:Ny
   
    %%%%%%%%%%%%%%%%%%%%%%%
    % Grab Diagonal Value %
    Diag = Y(Stage,Stage);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Substract All Previous Rows 
    % This will create a Colum of Zeros (Except for Diag)
    for Rows = 1:Stage - 1
        Y(Rows,:) = Y(Rows,:)*Diag - Y(Stage,:) * Y(Rows,Stage);
    end
end   
    
end

