%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief: Calculates the Forward-Gaussian Elimination
% This method can be used for Matrix Inversion,or direct solving of equations
% Basically creates a result in the form of:
% D U U U U
% 0 D U U U
% 0 0 D U U
% 0 0 0 D U
% 0 0 0 0 D
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Y ] = GaussElimForw( X )

%%%%%%%%%%%%%%%%%%%%%%%
% Grab Size of Matrix %
[Ny,~] = size(X);
Y = X;
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Forward Gaussian Elimination % 
for j = 1:Ny 
    C_ROW    = Y(j,:);
    Diag_Val = Y(j,j);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Subtract Rows %%%%%%% 
    for i = j + 1:Ny
        E_ROW = Y(i,:); % Currently Processed Row
        E_DIA = Y(i,j); % First Valid Value of the Row
        Y(i,:) = (E_ROW * Diag_Val - C_ROW * E_DIA) / Diag_Val;
    end
end

end


