%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief: Calculates the TriDiagonalization with Hauserholder's Algorithm
% --> Numerical Analysis by Burden and Faires <--
% For More Informatio,Please visit https://en.wikipedia.org/wiki/Householder_transformation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Y ] = HauseholderTriang( X )

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grab Matrix Dimension %
[Ny,~] = size(X);
Y = X;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference Matlab Implemntation of Hauseholder TriDiagonalization %
for i = 1:Ny - 1
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Select subsection of the original Matrix %
    AP = Y(i:end,i:end);
    [Nyi,~] = size(AP);
    
    a21 = AP(2,1);
    ai = -sign(a21)*sqrt(sum( (AP(2:end,1)).^2 ));
    ri = sqrt(0.5 * (ai^2 -a21*ai));

    %%%%%%%%%%%%%%%%%%%%%%
    % For Debugging Only %
    %s_a21 = sprintf('%7.4f',a21);
    %s_ai  = sprintf('%7.4f',ai);
    %s_ri  = sprintf('%7.4f',ri);
    %disp(['A21: ',s_a21,' AI: ',s_ai,' RI: ',s_ri]);
    
    Vi = single(zeros(Nyi,1));
    Vi(1) = 0;
    Vi(2) = (a21 - ai)/(2*ri);
    for j = 3 :Nyi
        Vi(j) = AP(j,1)/(2*ri);
    end
    
    %%%%%%%%%%%%%%%%%
    % Precompute Px %
    % Identity - 2xVector[Column]-Vector[Row] Multiply
    Px = single(eye(Nyi)) - 2 * (Vi*Vi');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Matrix-Matrix Multiply %
    APX = Px*AP*Px;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Original Matrix with New Values %
    Y(i:end,i:end) = APX;
    
end
end

