%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jacobi Solver: Solves a system of Equations
% A         : Matrix of Equalitions
% B         : Right Side
% X0        : Initial Seed (Guess)
% NumIter   : Maximum Amount of Iterations
% Precision : Required Precision for Convergence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Res ] = JacobiSolver( A,B,X0,NumIter,Precision )

%%%%%%%%%%%%%%%%%%%%%%%%
% Get Matrix Dimension %
[N,~] = size(A);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Diagonal Matrix (Out of A):
% D 0 0 0
% 0 D 0 0
% 0 0 D 0
% 0 0 0 D
D = (ones(N,1)*diag(A)').*eye(N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Residuum Matrix (Out of A) (R = L + U):
% L ... Matrix Below Diagonal
% U ... Matrix abbove Diagonal
% 0 U U U
% L 0 U U
% L L 0 U
% L L L 0
R = A - D;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Iteration Loop %%%%%%%
for i = 1:NumIter
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Jacobi Iteration %%%%%%%
    X1 = inv(D)*(B - R*X0);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Find Maximum Difference After Each Iteration % 
    Res = sum(sqrt((X1 - X0).^2));
    Res = max(Res(:));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Convergence Checking %%%%%%%
    if(Res < Precision)
        disp(['Iterations passed: ',num2str(i)]);
        X0 = X1;
        break;
    end
    
    X0 = X1;
end

Res = X0;


end

