%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief: Implements the Super-Relaxation Solver for Linear Systems.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Res ] = SuccessiveOverRelax( A,B,G0,NumIter,Omega,Precision )

[N,~]  = size(A);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Decompose into L|D|U %%%%%%

ii = 1:N;ii = ii';
ii = repmat(ii,[1,N]);
jj = 1:N;jj = repmat(jj,[N,1]);
IndU = ii < jj;
IndL = ii > jj;
D = diag(diag(A));  % ONLY DIAGONAL
U = A.*IndU;        % UPPER TRIANGULAR
L = A.*IndL;        % LOWER TRIANGULAR

T1 = (D + Omega*L)^(-1);
T2 = (-Omega*U + (1 - Omega)*D);
T3 = (Omega * B);

for i = 1:NumIter

    %%%%%%%%%%%%%%%%%%%%%%%%
    % SuperRelaxation Step %
    G1 = T1 * (T2*G0 + T3);
    
    %%%%%%%%%%%%%%%%%%%%%
    % Check Convergence %
    Diff = sum(sqrt((G0 - G1).^2));
    if(Diff < Precision)
        G0 = G1;
        disp(['Matlab SOR - Convergence after: ',num2str(i),' Iterations']);
        break;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Intermediate Result %
    G0 =  G1;
end
Res = G0; 


end

