#pragma once

// CUDA INCLUDES
#include "cufft.h"
#include "cuda_runtime.h"
#include "device_launch_parameters.h"

#include "DataTypes.h"

using namespace QNSwift;


// GET RID OF VISUAL STUDIO CUDA SYNCTHREAD UNDERLINED ERROR :)
#ifdef __INTELLISENSE__
void __syncthreads();
#endif

extern void CallMemSetReal1D	( LaunchParams Lau, cufftReal *Data, cufftReal Value, int Nx);
extern void CallMemSetReal2D	( LaunchParams Lau, cufftReal *Data, cufftReal Value, int Nx, int Ny);
extern void CallMemSetReal3D	( LaunchParams Lau, cufftReal *Data, cufftReal Value, int Nx, int Ny, int Nz);
extern void CallMemRewriteReal1D( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx);
extern void CallMemRewriteReal2D( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny);
extern void CallMemRewriteReal3D( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny, int Nz);

extern void CallMatrixAddMatrices				( LaunchParams Lau, cufftReal Alpha, cufftReal *IMatrix1, cufftReal Beta, cufftReal *IMatrix2, cufftReal Gamma, cufftReal *OMatrix, int Nx, int Ny );
extern void CallMatrixScalarMultiply			( LaunchParams Lau, cufftReal *IMatrix, cufftReal Value, int Nx, int Ny );
extern void CallMatrixCreateIdentity			( LaunchParams Lau, cufftReal *IMatrix, int Nx, int Ny );
extern void CallMatrixCreateExtendedIdentity	( LaunchParams Lau, cufftReal *IMatrix,cufftReal *OMatrix, int SingleNx, int Ny);
extern void CallMatrixCreateIdentityWithOffset	( LaunchParams Lau, cufftReal *IMatrix, int XOffset, int YOffset, int Nx, int Ny );
extern void CallMatrixInverseValues				( LaunchParams Lau, cufftReal *IMatrix, int Nx, int Ny );
extern void CallMatrixSelectDiagonalToVector_X	( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OVector, int Nx, int Ny);
extern void CallMatrixSelectDiagonalToVector_Y	( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OVector, int Nx, int Ny );
extern void CallMatrixSelectDiagonalToMatrix	( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int Nx, int Ny );
extern void CallMatrixSelectLowerTriangular		( LaunchParams Lau, cufftReal *IMatrix, cufftReal*OMatrix, int Nx, int Ny );
extern void CallMatrixSelectUpperTriangular		( LaunchParams Lau, cufftReal *IMatrix, cufftReal*OMatrix, int Nx, int Ny );
extern void CallMatrixOverwriteDiagonal			( LaunchParams Lau, cufftReal *IMatrix, cufftReal Value, int Nx, int Ny );
extern void CallMatrixSelectColumn_X			( LaunchParams Lau, cufftReal *IMatrix, cufftReal * OVector, int Nx, int Ny, int Column,int ColumnOffset);
extern void CallMatrixSelectColumn_Y			( LaunchParams Lau, cufftReal *IMatrix, cufftReal * OVector, int Nx, int Ny, int Column, int ColumnOffset );
extern void CallMatrixSelectRow					( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OVector, int Nx, int Ny, int Row,int RowOffset);
extern void CallMatrixTranspose					( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int Nx, int Ny);
extern void CallVectorColumRowMultiply			( LaunchParams Lau, cufftReal *ColumnVector, cufftReal *RowVector, cufftReal *OMatrix, int N);
extern void CallVectorAbs						( LaunchParams Lau, cufftReal * IVector, int N );
extern void CallVectorScalarMultiply			( LaunchParams Lau, cufftReal *IVector, cufftReal Value, int N );
extern void CallVectorAddVectors				( LaunchParams Lau, cufftReal Alpha, cufftReal* Ivector1, cufftReal Beta, cufftReal *IVector2, cufftReal Gamma, cufftReal *OVector, int N );

extern void CallMatrixMatrixMultiplyGlobal( LaunchParams Lau, cufftReal *MatrixIN1, cufftReal *MatrixIN2, cufftReal *MatrixOUT, int Nx, int Ny);
extern void CallMatrixMatrixMultiplyShared( LaunchParams Lau, cufftReal *MatrixIN1, cufftReal *MatrixIN2, cufftReal *MatrixOUT, int Nx, int Ny);
extern void CallMatrixVectorMultiplyGlobal( LaunchParams Lau, cufftReal *IMatrix, cufftReal *IVector, cufftReal *OVector, int Nx, int Ny );
extern void CallMatrixVectorMultiplyShared( LaunchParams Lau, cufftReal *IMatrix, cufftReal *IVector, cufftReal *OVector, int Nx, int Ny );

extern void CallMatrixGaussElimForward	( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int Nx, int Ny, int Iter);
extern void CallMatrixGaussElimBackWard	( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int Nx, int Ny, int Iter);
extern void CallMatrixSelectPiece		( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int XOffset, int YOffset, int NewSizeX, int NewSizeY, int Nx, int Ny);
extern void CallMatrixWritePiece		( LaunchParams Lau, cufftReal *IMatrix, cufftReal *OMatrix, int XOffset, int YOffset, int XOldSize, int YOldSize, int Nx, int Ny);
extern void CallMatrixInverseFinish		( LaunchParams Lau, cufftReal *IMatrix, int SingleNx, int Ny );
extern void CallMatrixRewriteWithOffset	( LaunchParams Lau, cufftReal *Destination, cufftReal * Source, int Nx_S, int Ny_S, int Nx_D, int Ny_D, int XOffset, int YOffset );
extern void CallVectorSum				( LaunchParams Lau, cufftReal *MainVector, cufftReal *UVectNormResults, int Nx, int Index, bool SaveLocally);
extern void CallVectorSubtract			( LaunchParams Lau, cufftReal *IVector, cufftReal *SVector, cufftReal *OVector, int N );
extern void CallRow2Col2D				( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny);
extern void CallCol2Row2D				( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny);
extern void CallRow2Col3D				( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny, int Nz);
extern void CallCol2Row3D				( LaunchParams Lau, cufftReal *IData, cufftReal *OData, int Nx, int Ny, int Nz);
extern void CallQR_MaltrixColumnMulps	( LaunchParams Lau, cufftReal *Matrix_A, cufftReal *Matrix_U, cufftReal * OutMat, int Nx, int Ny, int Iter);
extern void CallQR_SumMatrixAlongY		( LaunchParams Lau, cufftReal *IMatrix, cufftReal * VectorResults, int Nx, int Ny);
extern void CallQR_Finish				( LaunchParams Lau, cufftReal *Matrix_U, cufftReal *UResults, cufftReal *UAResults, int Nx, int Ny, int Iter);
extern void CallQR_CalculateQ			( LaunchParams Lau, cufftReal *Matrix_U, cufftReal *Matrix_Q, cufftReal * UVectSums, int Nx, int Ny);
extern void CallQR_PowerVector			( LaunchParams Lau, cufftReal *Vec, int Nx);
extern void CallHH_SingleParams			( LaunchParams Lau, cufftReal *Matrix, cufftReal *VecResults, cufftReal *A21, cufftReal *Ai, cufftReal *Ri, int Nx);
extern void CallHH_CreateVi				( LaunchParams Lau, cufftReal *Vi, cufftReal *Matrix, cufftReal *A21, cufftReal *Ai, cufftReal *Ri, int N);
extern void CallHH_CreateIdentity		( LaunchParams Lau, cufftReal *Matrix_IN, cufftReal* Matrix_OUT, int N);

