#pragma once
/*
 * WHOLE LIBRARY IS WRITTEN IN ROW-MAJOR ORDER!
 * TODO: VECTOR FOR MANAGING DEVICE MEMORY
 * TODO: VECTOR INPUTS, REMOVE "HOST" & "DEV" 
 */


// -----------------------------
// ------- CUDA Includes -------
#include "cublas.h"
#include "cublas_v2.h"
#include <cufft.h>
#include "cuda_runtime.h"
#include "device_launch_parameters.h"

// ----------------------------
// ------- C++ Includes -------
#include <stdio.h>
#include <vector>

// --------------------------------
// ------- Library includes -------
#include "DataTypes.h"
#include "MemEngine.h"
#include "Kernels.h"

using namespace std;

namespace QNSwift {
	class QSwift{
	private:

		// CLASS VARIABLES
		float Precision;
		bool EnablePrecision;
		int NumIter;
		cufftReal QS_Alpha;
		cufftReal QS_Beta;
		cufftReal QS_Gamma;
		cufftReal QS_Delta;
		cufftReal QS_Omega;
		ComputingType LibType;
		bool RuntimeECC;
		int RuntimeVersion;
		int DriverVersion;
		int NumDevices;
		float ElapsedTime;
		size_t TotalMem;
		size_t FreeMem;
		cudaEvent_t Start;
		cudaEvent_t Stop;
		cudaDeviceProp Prop;
		cublasHandle_t Blas;
		vector<IOArray> HostInputBuffer;
		vector<IOArray> HostOutputBuffer;
		MemEngine MemEng;

	
		// -----------------------------------------------
		// ------- INTERNAL FUNCTIONS AND ROUTINES -------
		static int GetNumCUDACores(int Major, int Minor, int SM);
		bool Handle_Error( cudaError_t err, const char *file, int line );
		bool Handle_CuFFT( cufftResult_t error, const char *file, int line );
		bool Handle_Cublas( cublasStatus_t error, const char *file, int line );
		void CalculateEigenValues( cufftReal * Matrix_A, int Nx, int Ny );
		void CalculateQRDec( cufftReal * Matrix_A, cufftReal *Matrix_Q, cufftReal *Matrix_R, int Nx, int Ny );
		void CalculateHHDec( cufftReal *Matrix_Data, int Nx, int Ny );
		void CalculateInverse( cufftReal *Matrix_Data , int Nx,int Ny);
		void GaussElimForward( cufftReal *Matrix_Data, cufftReal* Help_Matrix,int Nx,int Ny);
		void GaussElimBackward( cufftReal *Matrix_Data, cufftReal* Help_Matrix, int Nx,int Ny );
		void JacobiLinearSolver( cufftReal *Matrix_Data, cufftReal *RightSide, cufftReal *InitialGuess, int Nx, int Ny );
		void SuccesiveOverRelaxation( cufftReal *Matrix_Data, cufftReal *RightSide, cufftReal *InitialGuess, int Nx, int Ny );
		void CublasMMWrapperRM2CM( cufftReal *dev_Matrix_A, cufftReal *dev_Matrix_B, cufftReal *dev_Matrix_C, int Nx1, int Ny1, int Nx2, int Ny2 );

		// ------------------------------------------------
		// ------- ------- MEMORY FUNCTIONS ------- -------
		static void MemSafeUnregister( cufftReal *HostPtr, size_t Size );
		static void MemSafeRegister( cufftReal *HostPtr, size_t Size, unsigned int flag );

	
	public:
		DLLEXPORT bool Init();
		DLLEXPORT void SetParameter( int Value, ParameterType Type );
		DLLEXPORT void SetParameter( cufftReal Value, ParameterType Type);
		DLLEXPORT void SetInput( vector<IOArray>  Input );
		DLLEXPORT void SetOutput( vector<IOArray> Output);
		DLLEXPORT void SetComputingType(ComputingType Type);
		DLLEXPORT vector<string> GetErrorBuffer();
		DLLEXPORT vector<string> GetMessageBuffer();
		DLLEXPORT bool Compute();

		DLLEXPORT QSwift();  // Constructor
		DLLEXPORT ~QSwift(); // Destructor
		
	};
}

